# Overrides compiler.transformer (which constructs AST nodes out of a
# parse tree). Adds parse tree elements to the relevant ast nodes This
# is useful because matching the scanner tokens to the parse tree is
# easier than matching them to the AST nodes.

#from extended_ast import *
from compiler.transformer import Transformer as TransformerBase
import compiler

#from compiler.ast import Printnl, Print

import tokenutils
import tokenize
import token
import addtokens

def parse(src):
    return Transformer().parse(src)

Base = TransformerBase
class Transformer(TransformerBase):
    
    def parsesuite(self, text):
        return TransformerBase.parsesuite(self, text)

    def pass_stmt(self, node):
        n = Base.pass_stmt(self,node)
        n.parsetree = node
        return n

    def classdef(self, node):
        n = Base.classdef(self,node)
        n.parsetree = node
        return n

    def funcdef(self, node):
        n = Base.funcdef(self,node)
        n.parsetree = node
        return n


    def atom(self,node):
        n = Base.atom(self,node)
        n.parsetree = node
        #print "atom",n,"list:",node
        return n

    def import_stmt(self,node):
        n = Base.import_stmt(self,node)
        n.parsetree = node
        return n
        

    """

    # handles module
    def compile_node(self,node):   
        n = Base.compile_node(self,node)
        n.parsetree = node
        #print "compile_node",n,"list:",n.parsetree
        return n

    # handles Stmt, Function, Class, Discard
    def com_node(self,node):
        def addParsetreeToNodeIfItHasntAlreadyGotOne(node,parsetree):
            try:
                node.parsetree
            except AttributeError:
                node.parsetree = parsetree
                
        n = Base.com_node(self,node)
        addParsetreeToNodeIfItHasntAlreadyGotOne(n,node[1:])
        #print "com_node",n,"list:",n.parsetree
        return n
    
    """
    # handles getAttr, callfunc
    def com_apply_trailer(self, primaryNode, parsetree):
        n = Base.com_apply_trailer(self, primaryNode, parsetree)
        n.parsetree = primaryNode.parsetree + parsetree
        #print "com_apply_trailer",n,"list:",n.parsetree,"primary node:",primaryNode
        return n

    def print_stmt(self,nodelist):
        node = Base.print_stmt(self,nodelist)
        if isinstance(node,compiler.ast.Print):
            return Print(node.nodes,node.dest)
        elif isinstance(node,compiler.ast.Printnl):
            return Printnl(node.nodes,node.dest)
        else:
            return node


    def parse(self,src):
        ast = self.parsesuite(src)
        return ast


# Print.getChildNodes() reverses the order of the nodes and
# destination, which buggers up the token matching. This class
# fixes that
class Printnl(compiler.ast.Printnl):

    def getChildNodes(self):
        nodes = []
        nodes.extend(compiler.ast.flatten_nodes(self.nodes))
        if self.dest is not None:
            nodes = [self.dest] + nodes
        return tuple(nodes)
        

# Print.getChildNodes() reverses the order of the nodes and
# destination, which buggers up the token matching. This class
# fixes that
class Print(compiler.ast.Print):

    def getChildNodes(self):
        nodes = []
        nodes.extend(compiler.ast.flatten_nodes(self.nodes))
        if self.dest is not None:
            nodes = [self.dest] + nodes
        return tuple(nodes)
