;
;   XEmacs support for bicycle repair man
;   -------------------------------------
;   Author: Phil Dawes (pdawes@users.sourceforge.net)
;
;   Eval this buffer, then do 'M-x brm-load' to load a python package
;   structure into bicycle repair man. Then do 'M-x brm-rename' to
;   rename a class/function/method at point.
;
;   If you're debugging, check the 'Bicycle Repair Man' buffer for trace

(defun brm-start ()
"starts the bicycle repair man process"
  (make-comint "Bicycle Repair Man" "bikeemacs.py")
  (set-process-filter (get-process "Bicycle Repair Man") `brm-process-filter) 
)

(defun brm-kill ()
"kills the current bicycle repair man process"
  (interactive)
  (if (get-process "Bicycle Repair Man")
	  (kill-buffer (process-buffer (get-process "Bicycle Repair Man"))))
)

(defun brm-load (path)
"Loads the python file or directory (package) structure into bicycle repair man"
  (interactive "fPython file or directory: ")
  (brm-start)
  (process-send-string "Bicycle Repair Man" (concat "load " path "\n"))
)

(defun brm-rename (newname)
"Renames the class, function or method at point"
  (interactive "sNew name: ")
  (save-some-buffers)
  (process-send-string "Bicycle Repair Man" 
					   (format "rename %s %d %d %s\n" 
							   (buffer-file-name)
							   (current-column)
							   (count-lines 1 (point))
							   newname)))

(defun brm-get-type ()
"Returns a string representation of the type of the python construct \
at point"
  (interactive)
  (process-send-string "Bicycle Repair Man" 
					   (format "gettype %s %d %d\n" 
							   (buffer-file-name)
							   (current-column)
							   (count-lines 1 (point))))
  (accept-process-output (get-process "Bicycle Repair Man"))
  (setq myresult brm-result)
  (setq brm-result nil)
  myresult
)

(defun brm-test-get-type ()
  (interactive)
  (setq res (brm-get-type))
  (if res (message res) (message "Type wasnt recognised"))
)

(defvar brm-result nil)

(defun brm-process-filter (proc string)
"Internal filter for the bicycle repair man process"
  (save-excursion
	(set-buffer (process-buffer (get-process "Bicycle Repair Man")))
	(insert string))
  
  (if (string-match "BRMMessage: \\(.*\\)$" string)
	  (message (match-string 1 string)))
  (if (string-match "BRMUpdated: \\(.*\\)$" string)
	  (brm-revert-file-buffer (match-string 1 string)))
  (if (string-match "BRMResult: \\(.*\\)$" string)
	  (setq brm-result (match-string 1 string)))
  (if (string-match "BRMPromptForRename: \\([^ ]+\\) \\([^ ]+\\) \\([^ ]+\\) \\([^ ]+\\) \\([^ ]+\\)$" string)
	  (if (brm-prompt-for-rename (match-string 1 string)
								 (string-to-number (match-string 2 string))
								 (string-to-number (match-string 3 string))
								 (string-to-number (match-string 4 string))
								 (string-to-number (match-string 5 string))
								 "Couldn't deduce object type - rename this method reference? ")
		  (process-send-string "Bicycle Repair Man" "y\n")
		(process-send-string "Bicycle Repair Man" "n\n")))
  )	

(defun brm-prompt-for-rename (filename startx starty endx endy message)
"Internal function - goes to file and prompts user, asking if need to rename"
  (setq mybuf (current-buffer))
  (setq ans (save-excursion
			  (find-file filename)
			  (goto-line starty)
			  (move-to-column startx)
			  (set-mark (+ (point) (- endx startx)))
			  (activate-region)
			  (y-or-n-p message)))
  (switch-to-buffer mybuf)
  ans
  )

;(if (brm-prompt-for-rename "Couldn't deduce object type - rename this method reference? ")

;(

;(y-or-n-p "this is a prompt?")

;(brm-prompt-for-rename "/home/pld/src/bicyclerepair/bike/ui/scrap.py" "5" "3" "Couldn't deduce object type - rename this method reference? ")

;(progn (if (brm-prompt-for-rename "/home/pld/src/bicyclerepair/bike/ui/scrap.py" 5 3 "Couldn't deduce object type - rename this method reference? ")
;		   (message "Bicycle Repair Man y")
;		 (message "Bicycle Repair Man n")))


(defun brm-revert-file-buffer (filename)
"reverts the buffer currently looking at filename, if it exists"
  (setq mybuf (find-buffer-visiting filename))
  (if mybuf
	  (progn (set-buffer mybuf)
			 (revert-buffer nil 1))
	))


(defun brm-reload-file ()
  (if (string-match ".py$" (buffer-file-name))
	  (process-send-string "Bicycle Repair Man"		   
						   (format "load %s\n" (buffer-file-name))))
)


(add-hook 'after-save-hook 'brm-reload-file)


