#!/usr/bin/env python
import sys
if not "../.."in sys.path: sys.path.append("../..")

from bike.parsing import load
from bike.refactor import renameFunction, renameClass, renameMethod
from bike.parsing.output import save
from bike.parsing.utils import fqn_rcar, fqn_rcdr
from bike.parsing.addtypeinfo import addtypeinfo
from bike import bikefacade
import compiler
import compiler.ast
import os


class StandardIO:
    def write(self, output):
        import sys
        sys.stdout.write(output+"\n")
        #print output,
        sys.stdout.flush()

    def read(self):
        return raw_input()

class MuteIO(StandardIO):
    def write(self, output):
        pass

class ProgressWriter:
    def __init__(self, io):
        self.io = io

    def _write(self, txt):
        self.io.write(txt)

    def printDot(self, action, filename):
        import sys
        if not hasattr(self, "currentAction"):
            self._write(action)
        elif self.currentAction != action:
            self._write("\n"+action)
        self.currentAction = action
        self._write(".")

    def printFilename(self, action, filename):
        import sys
        self._write("BRMMessage: "+action+" "+filename)

class Server:
    def __init__(self, io = MuteIO()):
        self.io = io
        self.brmctx = None

    def inputLoop(self):
        while 1:
            input = raw_input()
            self.io.write("Input was :"+input)
            self.processCommand(input)

    def processCommand(self, cmd):
        command = cmd.split(" ")
        task = command[0]
        args = command[1:]
        if task == "load":
            self.load(args[0])

        if task == "rename":
            savedfiles = self.rename(args[0], int(args[1]), 
                                     int(args[2]), args[3])
            for file in savedfiles:
                self.io.write("BRMUpdated: "+file)

        if task == "reload":
            self.reloadSource(args[0])

    def load(self, path):
        if not self.brmctx:
            self.brmctx = bikefacade.load(path,ProgressWriter(self.io).printFilename)
        else:
            self.brmctx.load(path)
        self.io.write("BRMMessage: Bicycle Repair Man: code imported successfully")

    def rename(self, filename, x, y, newname):
        self.brmctx.setRenameMethodPromptCallback(self.promptUser)
        savedFiles = []
        try:
            self.brmctx.renameByCoordinates(filename,x,y,newname)
            savedFiles = self.brmctx.save()
        except bikefacade.CouldntLocateASTNodeFromCoordinatesException:
            self.io.write("BRMMessage: Couldn't find AST Node. Are you renaming the declaration?")
        self.io.write("done")
        return savedFiles

    def promptUser(self, filename, startcoords, endcoords):
        self.io.write("BRMPromptForRename: "+filename+ " " + \
                      str(startcoords.x)+" "+str(startcoords.y)+ " " + \
                      str(endcoords.x)+" "+str(endcoords.y))
        response = raw_input()
        self.io.write("BRMDebug response was "+response)
        if response == "y":
            return 1
        return 0

    def reloadSource(self, filename):
        self.brmctx.reloadSourceFile(filename)

    # this shouldnt be called, but is needed by the tests
    def _getAST(self):
        return self.brmctx._getAST()

if __name__ == "__main__":
    Server(StandardIO()).inputLoop()
